<?php

/**
 * Returns Contextly settings URL of specified type.
 *
 * @param string $type
 * @param null|string $access_token
 *   Is used to build direct login URL when specified.
 *
 * @return string
 */
function _contextly_settings_url($type, $access_token = NULL) {
  $query = array(
    'type' => $type,
  );

  if (isset($access_token)) {
    $query += array(
      'contextly_access_token' => $access_token,
    );
  }
  else {
    $query += array(
      'blog_url' => $GLOBALS['base_url'],
      'blog_title' => variable_get('site_name', 'Drupal'),
    );
  }

  return url(_contextly_server_url('cp') . 'redirect/', array('query' => $query));
}

/**
 * Settings form.
 */
function contextly_settings_form(&$state) {
  $form = array();
  $key = variable_get('contextly_api_key', array());

  $form['key'] = array(
    '#type' => 'fieldset',
    '#title' => t('API key'),
    '#collapsible' => TRUE,
    '#collapsed' => !empty($key),
  );

  $form['key']['link'] = array(
    '#type' => 'item',
    '#value' => t('In order to communicate securely, we use a shared secret key. You can find your secret API key on !home_link. Copy and paste it below.', array(
      '!home_link' => l(t("Contextly's settings panel"), _contextly_settings_url('tour'), array(
        'attributes' => array(
          'target' => '_blank',
        )
      )),
    )),
  );

  // We can't use #required here, because it would break the second submit
  // (which opens Contextly settings).
  $form['key']['contextly_api_key'] = array(
    '#type' => 'textfield',
    '#title' => t('Secret API key'),
    '#default_value' => implode('-', $key),
  );

  $form = system_settings_form($form);

  // Move "save" button to the API key fieldset and remove the rest. Also, move
  // the submit handler from the form to the button, to avoid its invocation on
  // the Contextly settings button click.
  $form['key']['submit'] = $form['buttons']['submit'];
  $form['key']['submit']['#submit'] = $form['#submit'];
  $form['key']['submit']['#validate'][] = 'contextly_settings_form_validate_key';
  unset($form['buttons'], $form['#submit']);

  $form['settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Service settings'),
    '#collapsible' => TRUE,
    '#collapsed' => empty($key),
  );
  $form['settings']['notice'] = array(
    '#type' => 'item',
    '#value' => t('The majority of the settings for Contextly are handled outside of Drupal. Click the settings button to securely login to your settings panel. If that fails, you can still <a href="@url" target="_blank">login via Twitter</a>.', array(
      '@url' => _contextly_settings_url('settings'),
    )),
  );
  $form['settings']['go'] = array(
    '#type' => 'submit',
    '#value' => t('Settings'),
    '#attributes' => array(
      'class' => 'contextly-settings-button',
    ),
    '#submit' => array('contextly_settings_form_submit_contextly_settings'),
  );

  drupal_add_js(drupal_get_path('module', 'contextly') . '/js/contextly-settings-form.js');

  return $form;
}

/**
 * Validation for the settings form.
 */
function contextly_settings_form_validate_key($form, &$state) {
  $key = trim($state['values']['contextly_api_key']);

  // Parse string key into an array.
  $key = explode('-', $key, 2);
  if (count($key) !== 2) {
    form_set_error('contextly_api_key', t('API key is incorrect.'));
    return;
  }

  // Convert it into associative array for later use.
  $key = array_combine(array('appID', 'appSecret'), $key);

  try {
    // Check API key. Create API client with isolated session.
    $api = new ContextlyApi(new ContextlySessionIsolated(), $key);

    // Failed authorization should throw an exception at this point.
    $api->connect();

    // Save parsed key back to values.
    $state['values']['contextly_api_key'] = $key;
  }
  catch (Exception $e) {
    _contextly_watchdog_exception($e);
    form_set_error('contextly_api_key', t('Test API request failed.'));
  }
}

/**
 * Submit handler for the "Contextly settings" button of the settings form.
 *
 * Redirects to Contextly service settings page.
 */
function contextly_settings_form_submit_contextly_settings($form, &$state) {
  // Try to get the auth token and use direct login.
  $token = NULL;
  try {
    $api = ContextlyApi::getInstance();
    $api->connect();
    $session = $api->getSession();
    $token = $session->getAccessToken();
  }
  catch (Exception $e) {
    // Just silently fall back to the Twitter login.
  }

  unset($_GET['destination']);
  $state['redirect'] = _contextly_settings_url('settings', $token);
}

/**
 * Contextly-enabled node types settings form.
 */
function contextly_settings_node_types_form(&$state) {
  $form = array();

  $types = node_get_types('names');
  $types = array_map('check_plain', $types);

  $form['types'] = array(
    '#type' => 'fieldset',
    '#title' => t('Enabled node types'),
  );

  // TODO Move to help
  $form['types']['info'] = array(
    '#type' => 'item',
    '#value' => t("Integration with Contextly will be enabled for selected node types only."),
  );

  $form['types']['contextly_all_node_types'] = array(
    '#type' => 'checkbox',
    '#title' => t('All node types.'),
    '#default_value' => variable_get('contextly_all_node_types', TRUE),
    '#attributes' => array(
      'class' => 'contextly-settings-use-all',
    ),
  );

  $form['types']['contextly_node_types'] = array(
    '#title' => t('Enabled types'),
    '#type' => 'checkboxes',
    '#options' => $types,
    '#default_value' => variable_get('contextly_node_types', array()),
    '#element_validate' => array('_contextly_multi_value_element_cleanup'),
    '#attributes' => array(
      'class' => 'contextly-settings-detailed',
    ),
  );

  drupal_add_js(drupal_get_path('module', 'contextly') . '/js/contextly-settings-form.js');

  return system_settings_form($form);
}

/**
 * Removes empty (not selected) values from multi-value form elements.
 */
function _contextly_multi_value_element_cleanup($element, &$state) {
  $filtered_value = array();
  if (!empty($element['#value'])) {
    $filtered_value = array_filter($element['#value']);
  }
  form_set_value($element, $filtered_value, $state);
}

function contextly_settings_tags_form($state) {
  $form = array();

  $form['tags'] = array(
    '#type' => 'fieldset',
    '#title' => t('Taxonomy terms'),
  );

  // TODO Move to help.
  $form['tags']['info'] = array(
    '#type' => 'item',
    '#value' => t('Terms from selected vocabularies will be sent to the Contextly.'),
  );

  $form['tags']['contextly_all_tags'] = array(
    '#type' => 'checkbox',
    '#title' => t('All taxonomy vocabularies'),
    '#default_value' => variable_get('contextly_all_tags', TRUE),
    '#attributes' => array(
      'class' => 'contextly-settings-use-all',
    ),
  );

  $node_types = node_get_types('names');
  foreach ($node_types as $type => $type_name) {
    $options = array();
    $vocabularies = taxonomy_get_vocabularies($type);
    foreach ($vocabularies as $vid => $vocabulary) {
      $options[$vid] = check_plain($vocabulary->name);
    }

    if (!empty($options)) {
      $form['tags']['contextly_tags__' . $type] = array(
        '#type' => 'checkboxes',
        '#title' => check_plain($type_name),
        '#options' => $options,
        '#default_value' => variable_get('contextly_tags__' . $type, array()),
        '#element_validate' => array('_contextly_multi_value_element_cleanup'),
        '#attributes' => array(
          'class' => 'contextly-settings-detailed',
        ),
      );
    }
  }

  drupal_add_js(drupal_get_path('module', 'contextly') . '/js/contextly-settings-form.js');

  return system_settings_form($form);
}
